from zygoat.components import FileComponent
from zygoat.constants import Projects

from . import resources


class Black(FileComponent):
    filename = "pyproject.toml"
    resource_pkg = resources
    base_path = Projects.BACKEND


black = Black()



from importlib import resources as il_resources
import logging
import os

from zygoat.constants import Projects, Phases
from zygoat.components import Component
from zygoat.utils.files import use_dir

from . import resources

log = logging.getLogger()

init_file = "__init__.py"
file_name = "wait_for_db.py"


# TODO: Refactor this later to avoid repetition in paths
class WaitCommand(Component):
    def _touch(self, path):
        if os.path.exists(path):
            return

        log.info(f"Creating {path}")
        open(path, "w").close()

    def create(self):
        with use_dir(Projects.BACKEND):
            log.info("Creating directory structure")
            os.makedirs(
                os.path.join(Projects.BACKEND, "management", "commands"), exist_ok=True
            )

            self._touch(os.path.join(Projects.BACKEND, "management", init_file))
            self._touch(os.path.join(Projects.BACKEND, "management", "commands", init_file))

            log.info("Copying command into project")
            with open(
                os.path.join(Projects.BACKEND, "management", "commands", file_name), "w"
            ) as f:
                f.write(il_resources.read_text(resources, file_name))

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)

    def delete(self):
        with use_dir(Projects.BACKEND):
            log.info("Deleting command file")
            os.remove(os.path.join(Projects.BACKEND, "management", "commands", file_name))

    @property
    def installed(self):
        with use_dir(Projects.BACKEND):
            return os.path.exists(
                os.path.join(Projects.BACKEND, "management", "commands", file_name,),
            )


wait_command = WaitCommand()



import logging
import os
import shutil

from zygoat.components import Component
from zygoat.constants import Projects
from zygoat.utils.shell import run

from .dockerfile import dockerfile
from .gitignore import gitignore
from .prettier import prettier
from .eslint import eslint
from .cypress import cypress
from .next_config import next_config
from .proxy import proxy
from .dependencies import dependencies

log = logging.getLogger()


class Frontend(Component):
    def create(self):
        log.info("Installing/upgrading yarn through npm")
        run(["npm", "install", "-g", "--upgrade", "yarn"])

        log.info("Running create-next-app")
        run(["yarn", "create", "next-app", Projects.FRONTEND, "-e"])

        log.info("Deleting a poorly formatted index page")
        open(os.path.join(Projects.FRONTEND, "pages", "index.js"), "w").close()

        log.info("Deleting the default api directory")
        shutil.rmtree(os.path.join(Projects.FRONTEND, "pages", "api"))

    def delete(self):
        log.warning(f"Deleting the {Projects.FRONTEND} project")
        shutil.rmtree(Projects.FRONTEND)

    @property
    def installed(self):
        return os.path.exists(Projects.FRONTEND)


frontend = Frontend(
    sub_components=[
        dockerfile,
        gitignore,
        prettier,
        dependencies,
        eslint,
        cypress,
        next_config,
        proxy,
    ]
)



import shlex
import subprocess


def run(cmd, *args, **kwargs):
    """
    Takes a shell command split into an array and executes it

    :param cmd: An iterable object with strings inside that form a command
    """
    return subprocess.run(
        " ".join([shlex.quote(c) for c in cmd]), *args, shell=True, check=True, **kwargs,
    )



import logging

from zygoat.components import FileComponent, Component
from zygoat.constants import Projects, Phases
from zygoat.utils.backend import install_dependencies

from . import resources

log = logging.getLogger()


class PytestConfig(FileComponent):
    filename = "pytest.ini"
    resource_pkg = resources
    base_path = Projects.BACKEND


class Pytest(Component):
    def create(self):
        dependencies = [
            "pytest",
            "pytest-django",
            "pytest-cov",
        ]

        log.info("Installing pytest dev dependencies")
        install_dependencies(*dependencies, dev=True)

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)


pytest = Pytest(sub_components=[PytestConfig()])



import os

from zygoat.constants import Projects
from zygoat.components import FileComponent

from . import resources


class Configuration(FileComponent):
    filename = "cypress.json"
    resource_pkg = resources
    base_path = os.path.join(Projects.FRONTEND)


configuration = Configuration()



import os

from redbaron import RedBaron

from zygoat.utils.files import repository_root
from zygoat.constants import Projects
from . import Component


class SettingsComponent(Component):
    ORIGINAL_FILE_NAME = "settings.py"
    DIRECTORY_NAME = "settings"
    MODULE_NAME = "zygoat_settings"
    FILE_NAME = f"{MODULE_NAME}.py"

    @property
    def initial_settings_file_path(self):
        return os.path.join(
            Projects.BACKEND, Projects.BACKEND, SettingsComponent.ORIGINAL_FILE_NAME
        )

    @property
    def settings_directory(self):
        return os.path.join(
            Projects.BACKEND, Projects.BACKEND, SettingsComponent.DIRECTORY_NAME,
        )

    @property
    def settings_file_path(self):
        return os.path.join(self.settings_directory, SettingsComponent.FILE_NAME,)

    def parse(self):
        with repository_root():
            with open(self.settings_file_path) as f:
                return RedBaron(f.read())

    def dump(self, data):
        with repository_root():
            with open(self.settings_file_path, "w") as f:
                f.write(data.dumps())



import logging

from zygoat.components import Component
from zygoat.constants import Phases
from zygoat.utils.backend import install_dependencies

log = logging.getLogger()


class Dependencies(Component):
    def create(self):
        dependencies = [
            "Django",
            "psycopg2-binary",
            "django-cors-headers",
            "djangorestframework",
            "django-environ",
            "djangorestframework-camel-case",
            "django-anymail",
        ]

        dev_dependencies = ["pytz", "factory-boy", "flake8-black", "bandit"]

        log.info("Installing production dependencies")
        install_dependencies(*dependencies)

        log.info("Installing dev dependencies")
        install_dependencies(*dev_dependencies, dev=True)

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)


dependencies = Dependencies()



from time import sleep

from django.core.management.base import BaseCommand
from django.db import connection
from django.db.utils import OperationalError


class Command(BaseCommand):
    def handle(self, *args, **kwargs):
        while True:
            sleep(0.5)
            try:
                with connection.temporary_connection():
                    self.stdout.write(self.style.SUCCESS("Connected to db."))
                    break
            except OperationalError:
                self.stdout.write(self.style.WARNING("Still waiting for db..."))
